###############################################################
# wsadmin script to deploy Emptoris Strategic Supply Management
###############################################################

########### Start Imports ###########
from install.GetProfilePath import GetProfilePath

from util.Boolean import *
import util.utility as util

import was_objs.import_was as wasScript
execfile(wasScript.__file__)
########### End Imports ###########

########### Start Variable Definitions ###########
wasHome = args['wasHome']
profileName = args['profileName']
serverName = args['serverName']
nodeName = args['nodeName']
cellName = args['cellName']
debugMode = args['debug']
debugPort = args['debugPort']
suiteApps = None
installIndexServerApplication = args['installIndexServerApplication']

########### End Variable Definitions ###########

# Make sure the WAS profile exists
profilePath = GetProfilePath(wasHome).getProfilePath(profileName)
if profilePath is None:
    raise Exception('Could not find target profile path for profile [' + profileName + ']')

try:
    print "Configuring IBM Emptoris Strategic Supply Management..."

    ########### Create a new server ###########
    svr = server.Server(serverName, nodeName, cellName)
    svr.setPortSet(args['portSet'])
    svr.configure()

    oracleDatabase = "Oracle"
    db2Database = "DB2"
    vendor = None
    if args['rdbmsVendor'] == oracleDatabase :
        varName = "ORACLE_JDBC_DRIVER_PATH"
        vendor = oracleDatabase
    else:
        varName = "DB2_JCC_DRIVER_PATH"
        vendor = db2Database
    newVarValue = args['jdbcDriverPath']
    nodeConfigId = AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName + '/')
    varSubstitutions = AdminConfig.list("VariableSubstitutionEntry",nodeConfigId).splitlines()
    for varSubst in varSubstitutions:
        getVarName = AdminConfig.showAttribute(varSubst, "symbolicName")
        if getVarName == varName:
            AdminConfig.modify(varSubst,[["value", newVarValue]])
            break
    print 'database = ' + vendor
    print  'url = '+ args['dataSourceUrl']

    ########### Start JVM Configuration ###########

    jvmconfig = jvm.JavaVirtualMachine(svr)
    jvmconfig.setInitialHeapSize(args['javaHeap'])
    jvmconfig.setMaximumHeapSize(args['javaHeap'])
    jvmconfig.setClasspath([args['gatewayClasspath']])
    jvmconfig.setDebugMode(debugMode)
    jvmconfig.setDebugArgs("-agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address=" + str(debugPort))

    jvmconfig.addSystemProperty('shared.code.install.dir', args['shared.code.install.dir'], 'SSM Shared Installation Directory')
    jvmconfig.addSystemProperty('file.encoding','UTF8','File default encoding')
    jvmconfig.addSystemProperty('rivermine.registries','com.rivermine.app.gateway.GatewayRegistry','SSM Application Registry')
    jvmconfig.addSystemProperty('rivermine.mode','prod','SSM Application start mode')
    jvmconfig.addSystemProperty('rivermine.server.port',str(args['rivermineServerPort']),'SSM Application server port')
    jvmconfig.addSystemProperty('rivermine.home',args['rivermineHome'],'SSM Application home location')
    jvmconfig.addSystemProperty('default.language',args['defaultLanguage'],'Category default language')
    jvmconfig.addSystemProperty('supplier.discoveryProviders','','Supplier discovery providers')
    jvmconfig.addSystemProperty('hibernate.wxs.catalog.service.bootstrap.port',str(args['wxsCatalogPort']),'Hibernate eXtreme Scale bootstrap port number')
    jvmconfig.addSystemProperty('rivermine.ecm.sharedSecret',args['encryptedSSOPassword'],'ECM Shared Secret')
    jvmconfig.addSystemProperty('ssm.indexserver.url',args['ssm.indexserver.url'],'URL of the Index Server')
    jvmconfig.addSystemProperty('ssm.indexserver.home',args['ssm.indexserver.home'],'Home Directory of the Index Server')
    jvmconfig.addSystemProperty('emptoris.sso.authentication.configuration.provider',args['sso.provider'],'Indicate a core or custom SSO provider')
    jvmconfig.addSystemProperty('emptoris.sso.authentication.type',args['sso.type'],'The type of SSO being deployed')

    if installIndexServerApplication == 'true':
        jvmconfig.addSystemProperty('solr.solr.home',args['ssm.indexserver.home'],'Index Server Home Directory')

    if 'ecmBackendUrl' in args.keys() and args['ecmBackendUrl'] is not None and args['ecmBackendUrl'] != "":
        jvmconfig.addSystemProperty('com.ibm.ecm.backend.url',args['ecmBackendUrl'],'URL of the ECM backend server')

    if 'espBackendUrl' in args.keys() and args['espBackendUrl'] is not None and args['espBackendUrl'] != "":
        jvmconfig.addSystemProperty('com.ibm.esp.backend.url',str(args['espBackendUrl']),'URL of the Sourcing backend server')

    if 'spendBackendUrl' in args.keys() and args['spendBackendUrl'] is not None and args['spendBackendUrl'] != "":
        jvmconfig.addSystemProperty('com.ibm.spend.backend.url',str(args['spendBackendUrl']),'URL of the Spend backend server')


    if args['source.appliance.supplied'] == 'true':
        jvmconfig.addSystemProperty('com.ibm.source.installation.label',args['source.installation.label'],'Installation label of the source system for the transport utility')
        jvmconfig.addSystemProperty('com.ibm.ssm.transport.source.location',args['ssm.transport.source.location'],'URL of the SSM source server')

    jvmconfig.addSystemProperty('rivermine.smtp.server',args['smtpServer'],'SMTP Mail Server')

    if args['antivirusEnabled'] == 'true':
        jvmconfig.addSystemProperty('rivermine.antivirus.server.host'," " + args['antivirusServerHost'],'Antivirus Server host name')
        jvmconfig.addSystemProperty('rivermine.antivirus.server.port',str(args['antivirusServerPort']),'Antivirus Server port number')

    if 'defaultCurrency' in args.keys() and args['defaultCurrency'] is not None and args['defaultCurrency'] != "":
        jvmconfig.addSystemProperty('rivermine.default.currency',args['defaultCurrency'],'Default Currency')

    doEnableAccess = 'enableSqlAndBeanshellAccess' in args.keys() and args['enableSqlAndBeanshellAccess'] is not None and args['enableSqlAndBeanshellAccess'] != "" and args['enableSqlAndBeanshellAccess']
    jvmconfig.addSystemProperty('rivermine.enablesqlaccess ',doEnableAccess,'Enable SQL Access')

    doEnableAccess = 'webservices.unify.faults' in args.keys() and args['webservices.unify.faults'] is not None and args['webservices.unify.faults'] != "" and args['webservices.unify.faults']
    jvmconfig.addSystemProperty('webservices.unify.faults',doEnableAccess,'Prevent Error Details in Web Service Messages')

    jvmconfig.addSystemProperty('ssm.web.url',args['ssm.web.protocol']+ '://'+ args['ssm.web.host']+ ':'+ str(args['ssm.web.port']) + '/emptoris','SSM Application URL')

    suiteApps = []
    if args['pgm.enabled'] == 'true':
        suiteApps.append('PGM')

    if args['suite.apps.slm'] == 'SLM':
        suiteApps.append('SLM')
        jvmconfig.addSystemProperty('slm.web.url',args['ssm.web.protocol']+ '://'+ args['ssm.web.host']+ ':'+ str(args['slm.web.port'])+ '/srm-app','SLM Application URL')

    if args['suite.apps.esp'] == 'ESP':
        suiteApps.append('ESP')
        jvmconfig.addSystemProperty('esp.web.url',args['ssm.web.protocol']+ '://'+ args['ssm.web.host']+ ':'+ str(args['esp.web.port'])+ '/sourcing','Sourcing Application URL')

    if args['suite.apps.ecm'] == 'ECM':
        suiteApps.append('ECM')
        jvmconfig.addSystemProperty('ecm.web.url',args['ssm.web.protocol']+ '://'+ args['ssm.web.host']+ ':'+ str(args['ecm.web.port'])+ '/dicarta','Contract Management Application URL')

    if args['suite.apps.spend'] == 'Spend':
        suiteApps.append('Spend')
        jvmconfig.addSystemProperty('spend.web.url',args['ssm.web.protocol']+ '://'+ args['ssm.web.host']+ ':'+ str(args['spend.web.port'])+ '/expensemap', 'Spend Application URL')

    if suiteApps is not None and len(suiteApps) > 0:
        jvmconfig.addSystemProperty('suite.apps', ','.join(suiteApps), 'Enabled Suite Applications')

    if args['configure.oauth.support'] == 'true':
        jvmconfig.addSystemProperty('oauth.provider.name',args['oauth.provider.name'])
        jvmconfig.addSystemProperty('oauth.client.secret',args['encryptedClientSecret'])
        jvmconfig.addSystemProperty('source.oauth.provider.name',args['source.oauth.provider.name'])
        jvmconfig.addSystemProperty('installation.label',args['installation.label'])

    if 'cognos.available' in args.keys() and args['cognos.available'] is not None and args['cognos.available'] == 'true':
        jvmconfig.addSystemProperty('cognos.web.url',args['ssm.web.protocol']+ '://'+ args['ssm.web.host']+ ':'+ str(args['ssm.web.port'])+ '/ibmcognos', 'Cognos Application URL')

    # SAML
    samlTrustStorePath = args['profilePath'] + '/config/cells/' + cellName + '/nodes/' + nodeName + '/trust.p12'
    samlWSSecurityKeystorePath = args['profilePath'] + '/config/vsm-ws-cert.jks'
    samlSpEntityId= 'https://' + args['domainName'] + '/emptoris/'
    jvmconfig.addSystemProperty('com.ibm.truststore.path',samlTrustStorePath,'SAML Trust Store File Path')
    jvmconfig.addSystemProperty('ssm.saml.signer.cert.alias','ssmsamlsignercert','SAML Signer Cert Alias')
    jvmconfig.addSystemProperty('ssm.saml.sp.entityid',samlSpEntityId,'SAML SP Entity ID')
    jvmconfig.addSystemProperty('ssm.saml.wssecurity.keystore.path',samlWSSecurityKeystorePath,'WS-Security Certificate keystore')
    
    
    #Disable DSF OWASP ESAPI Security Wrapper Filter 
    jvmconfig.addSystemProperty('com.ibm.emptoris.dsf.servlet.security.wrapper.enabled','false','Enable or disable OWASP ESAPI Security Filter')
    
    
    jvmconfig.configure()
    ########### End JVM Configuration ###########

    ########### Start DataSource Configuration ###########
    auth = jaasAuthData.JAASAuthData()
    auth.setAlias(args['dataSourceName'])
    auth.setDescription("Credentials for DefaultDS")
    auth.setUserId(args['dataSourceUserId'])
    auth.setPassword(args['dataSourcePassword'])
    auth.setServer(svr)
    auth.configure()

    jdbcP = jdbcProvider.JDBCProvider()
    jdbcP.setName(args['dataSourceName'])
    jdbcP.setVendor(args['rdbmsVendor'])
    jdbcP.setClasspath(args['jdbcDriverLocation'])
    jdbcP.setServer(svr)
    jdbcP.configure()

    ds = dataSource.DataSource()
    ds.setName(args['dataSourceName'])
    ds.setCategory(args['dataSourceName'])
    ds.setJndiName(args['dataSourceJndiName'])
    if 'dataSourceUrl' in args.keys() and args['rdbmsVendor'] == oracleDatabase :
        ds.setURL(args['dataSourceUrl'])
    else:
        ds.setResourceAttribute('databaseName', args['databaseServicename'])
        ds.setResourceAttribute('driverType', 4, 'java.lang.Integer')
        ds.setResourceAttribute('serverName', "" + args['databaseIP'])
        ds.setResourceAttribute('portNumber', args['databasePort'], 'java.lang.Integer')
    ds.setAuthDataAlias(auth)
    ds.setServer(svr)
    ds.setJdbcProvider(jdbcP)
    ds.setStatementCacheSize(100)
    ds.configure()
    datasourceConfigId = ds.getConfigId()
    connectionPoolId = AdminConfig.list('ConnectionPool', datasourceConfigId)
    AdminConfig.modify(connectionPoolId, '[[maxConnections "25"] [minConnections "5"]]')
    if args['rdbmsVendor'] == db2Database :
        ds.setResourceAttribute('currentSchema', args['db2Schema'])
        ds.setResourceAttribute('currentFunctionPath', args['db2Schema'])
        ds.configure()

    ########### End DataSource Configuration ###########

    ########### Start Second DataSource Configuration ###########
    auth = jaasAuthData.JAASAuthData()
    auth.setAlias(args['urDataSourceName'])
    auth.setDescription("Credentials for DefaultDS")
    auth.setUserId(args['dataSourceUserId'])
    auth.setPassword(args['dataSourcePassword'])
    auth.setServer(svr)
    auth.configure()

    jdbcP = jdbcProvider.JDBCProvider()
    jdbcP.setName(args['urDataSourceName'])
    jdbcP.setVendor(args['rdbmsVendor'])
    jdbcP.setClasspath(args['jdbcDriverLocation'])
    jdbcP.setServer(svr)
    jdbcP.configure()

    ds2 = dataSource.DataSource()
    ds2.setName(args['urDataSourceName'])
    ds2.setCategory(args['urDataSourceName'])
    ds2.setJndiName(args['urDataSourceJndiName'])
    if 'dataSourceUrl' in args.keys() and args['rdbmsVendor'] == oracleDatabase :
        ds2.setURL(args['dataSourceUrl'])
    else:
        ds2.setResourceAttribute('databaseName', args['databaseServicename'])
        ds2.setResourceAttribute('driverType', 4, 'java.lang.Integer')
        ds2.setResourceAttribute('serverName', args['databaseIP'])
        ds2.setResourceAttribute('portNumber', args['databasePort'], 'java.lang.Integer')
    ds2.setAuthDataAlias(auth)
    ds2.setServer(svr)
    ds2.setJdbcProvider(jdbcP)
    ds2.configure()
    
    ds2.setProperty("nonTransactionalDataSource", "true", propType="java.lang.Boolean")
    ds2.setProperty("commitOrRollbackOnCleanup", "rollback", propType="java.lang.String")

    datasourceConfigId = ds2.getConfigId()
    connectionPoolId = AdminConfig.list('ConnectionPool', datasourceConfigId)
    AdminConfig.modify(connectionPoolId, '[[maxConnections "10"] [minConnections "1"]]')
    if args['rdbmsVendor'] == db2Database :
        ds2.setResourceAttribute('currentSchema', args['db2Schema'])
        ds2.setResourceAttribute('currentFunctionPath', args['db2Schema'])
        ds2.configure()

    ########### End Second DataSource Configuration ###########

    ########### Start Library Configuration ###########
    sharedLib = library.Library("Library")
    sharedLib.setDescription('Shared library for Gateway')
    sharedLib.setScope(svr.getCellContainmentPathId())
    jsonFileLocation='${WAS_INSTALL_ROOT}/plugins/org.apache.tuscany.sdo.sca.core.model/JSON4J.jar'
    sharedLib.setClassPath(args['appLibLocation'] + util.getClasspathSeparator() + jsonFileLocation)
    sharedLib.configure()
    ########### End Library Configuration ###########

    ########### Start login entry ###########
    gatewayLoginEntry = jaasLoginEntry.JAASLoginEntry()
    gatewayLoginEntry.setLoginEntryAlias('Gateway')
    gatewayLoginEntry.setLoginType('application')

    # SSOClientLoginModule
    #ssoLoginModule = jaasAppLoginModule.JAASApplicationLoginModule()
    #ssoLoginModule.setLoginEntryAlias('Sso')
    #ssoLoginModule.setLoginModule('com.emptoris.framework.remote.client.sso.SSOClientLoginModule')
    #ssoLoginModule.setAuthStrategy('REQUIRED')
    #ssoLoginModule.addCustomProperty('debug','false')

    ########### Start Application Deployment ###########
    domainName = ""
    if 'domainName' in args.keys() and args['domainName'] is not None:
        domainName = args['domainName']

    inst = installer.Installer(args['appFileLocation'], args['installerPropertiesLocation'])
    inst.setServer(svr)

    inst.setMapSharedLibForMod([[".*", 'META-INF/application.xml', sharedLib.getName()]])
    inst.setProcessEmbeddedConfig(false)
    inst.setNoprocessEmbeddedConfig(true)
    inst.setMapAuthenticatedUsersToAllAuthenticatedRealms(true)
    inst.setCookieDomain(domainName)

    if 'overlayPath' in args.keys() and args['overlayPath'] is not None and not args['overlayPath'] == '':
        inst.setOverlay(args['overlayPath'])

    # Check if we need to remove old installation
    # This is needed if the version is prior to 10.0.1.0 when we started packaging an ear file
    # Get the version info of the already-installed application
    print "Getting the installed version."
    inst.setAppname('emptoris')
    if inst.isInstalled():
        print "emptoris is installed"
        installedBuildVersion = AdminApp.view('emptoris', '-buildVersion')
        installedBuildVersion = installedBuildVersion.split(':', 1)[1].strip()
        print "Current build is " + installedBuildVersion
        print "uninstall is needed."
        inst.uninstall()

    inst.configure()

    # Set the start order of VSM to be after the index server
    AdminConfig.modify(AdminConfig.showAttribute(AdminConfig.getid("/Deployment:emptoris/"),"deployedObject"),[['startingWeight','5']])

    print "Saving changes..."
    AdminConfig.save()
    print "Changes saved."

    print "Configuring emptoris classloader"
    app = AdminConfig.getid('/Deployment:emptoris/')

    appObject = AdminConfig.showAttribute(app, "deployedObject")
    AdminConfig.modify(appObject, [['warClassLoaderPolicy', 'SINGLE']])

#    classloader = AdminConfig.showAttribute(appObject, 'classloader')
#    AdminConfig.modify(classloader, [['mode', 'PARENT_LAST']])
    
    print "Saving changes..."
    AdminConfig.save()
    print "Changes saved." 

    # Install the index server app
    if installIndexServerApplication == 'true':
        inst = installer.Installer(args['indexServerAppFileLocation'])
        inst.setServer(svr)
        inst.setAppname(args['indexServerAppName'])
        inst.configure()
        print "Saving changes..."
        AdminConfig.save()
        print "Changes saved."

        ########### Modifying WAR class loader mode using wsadmin scripting ##########
        deployments = AdminConfig.getid('/Deployment:SSMIndexServer/')
        deploymentObject = AdminConfig.showAttribute(deployments, 'deployedObject')
        myModules = AdminConfig.showAttribute(deploymentObject, 'modules')
        myModules = myModules[1:len(myModules)-1].split(" ")
        print myModules
        for module in myModules:
            if (module.find('WebModuleDeployment')!= -1):
                AdminConfig.modify(module, [['classloaderMode', 'PARENT_LAST']])
        AdminConfig.save()

    ########### End Application Deployment ###########
except Exception, e:
    print "Server configuration failed.  Rolling back changes."
    AdminConfig.reset()
    raise # Re-throw to terminate the script

