###############################################################
# wsadmin script to deploy the SSM Index Server
###############################################################

########### Start Imports ###########
from install.GetProfilePath import GetProfilePath

from util.Boolean import * 
import util.utility as util

import was_objs.import_was as wasScript
execfile(wasScript.__file__)
########### End Imports ###########

########### Start Variable Definitions ###########
wasHome = args['wasHome']
profileName = args['profileName']
serverName = args['serverName']
nodeName = args['nodeName']
cellName = args['cellName']

########### End Variable Definitions ###########

# Make sure the WAS profile exists
profilePath = GetProfilePath(wasHome).getProfilePath(profileName)
if profilePath is None:
    raise Exception('Could not find target profile path for profile [' + profileName + ']') 

try:
    print "Configuring IBM Emptoris SSM Index Server..."

    ########### Create a new server ###########
    svr = server.Server(serverName, nodeName, cellName)
    
    svr.setPortSet(args['portSet'])
    svr.configure()
    
    oracleDatabase = "Oracle"
    db2Database = "DB2"

    ########### Start JVM Configuration ###########
        
    jvmconfig = jvm.JavaVirtualMachine(svr)

    jvmconfig.setInitialHeapSize(args['javaHeap'])
    jvmconfig.setMaximumHeapSize(args['javaHeap'])
    
    #server = AdminConfig.getid('/Server:'+sys.argv[0]+'/')
    #jvm = AdminConfig.list('JavaVirtualMachine', server)

    jvmconfig.addSystemProperty('solr.solr.home',args['indexServerHome'],'Index Server Home Directory')
    
    jvmconfig.configure()
    ########### End JVM Configuration ###########
    
    ########## Create the datasource ##########
    urDataSourceName = args['urDataSourceName']
    urDataSourceJndiName = args['urDataSourceJndiName']
    ssmDataSourceUserId = args['ssmDataSourceUserId']
    ssmDataSourcePassword = args['ssmDataSourcePassword']
    ssmRdbmsVendor = args['ssmRdbmsVendor']
    ssmJdbcDriverLocation = args['ssmJdbcDriverLocation']
    ssmDataSourceUrl = args['ssmDataSourceUrl']
    ssmDb2Schema = args['ssmDb2Schema']
    ssmJdbcDriverPath = args['ssmJdbcDriverPath']
    ssmDatabaseServicename = args['ssmDatabaseServicename']
    ssmDatabaseIP = args['ssmDatabaseIP']
    ssmDatabasePort = args['ssmDatabasePort']

    auth = jaasAuthData.JAASAuthData()
    auth.setAlias(urDataSourceName)
    auth.setDescription("Credentials for  SSM DS")
    auth.setUserId(ssmDataSourceUserId)
    auth.setPassword(ssmDataSourcePassword)
    auth.setServer(svr)
    auth.configure()

    jdbcP = jdbcProvider.JDBCProvider()
    jdbcP.setName(urDataSourceName)
    jdbcP.setVendor(ssmRdbmsVendor)
    jdbcP.setClasspath(ssmJdbcDriverLocation)
    jdbcP.setServer(svr)
    jdbcP.configure()

    ds = dataSource.DataSource()
    ds.setName(urDataSourceName)
    ds.setDescription('Data source for Index Server connection to SSM')
    ds.setCategory(urDataSourceName)
    ds.setJndiName(urDataSourceJndiName)
    if 'ssmDataSourceUrl' in args.keys() and ssmRdbmsVendor == oracleDatabase :
        ds.setURL(ssmDataSourceUrl)
    else:
        ds.setResourceAttribute('databaseName', ssmDatabaseServicename)
        ds.setResourceAttribute('driverType', 4, 'java.lang.Integer')
        ds.setResourceAttribute('serverName', ssmDatabaseIP)
        ds.setResourceAttribute('portNumber', ssmDatabasePort, 'java.lang.Integer')
    ds.setAuthDataAlias(auth)
    ds.setServer(svr)
    ds.setJdbcProvider(jdbcP)
    ds.configure()
    
    ds.setProperty("nonTransactionalDataSource", "true", propType="java.lang.Boolean")
    
    datasourceConfigId = ds.getConfigId()
    connectionPoolId = AdminConfig.list('ConnectionPool', datasourceConfigId)
    AdminConfig.modify(connectionPoolId, '[[maxConnections "10"] [minConnections "5"]]')
    if ssmRdbmsVendor == db2Database :
        ds.setResourceAttribute('currentSchema', ssmDb2Schema)
        ds.setResourceAttribute('currentFunctionPath', ssmDb2Schema)
        ds.configure()
    ########### End DataSource Configuration ###########
    
    ########### Start Application Deployment ###########
    inst = installer.Installer(args['appFileLocation'])
    inst.setServer(svr)
    inst.setAppname(args['appName'])
    inst.configure()
    print "Saving changes..."
    AdminConfig.save()
    print "Changes saved."
    ########### End Application Deployment ###########
    
    ########### Modifying WAR class loader mode using wsadmin scripting ##########
    deployments = AdminConfig.getid('/Deployment:SSMIndexServer/')
    deploymentObject = AdminConfig.showAttribute(deployments, 'deployedObject')
    myModules = AdminConfig.showAttribute(deploymentObject, 'modules')
    myModules = myModules[1:len(myModules)-1].split(" ")
    print myModules
    for module in myModules:
        if (module.find('WebModuleDeployment')!= -1):
            AdminConfig.modify(module, [['classloaderMode', 'PARENT_LAST']])

    AdminConfig.save()


except Exception, e:
    print "Server configuration failed.  Rolling back changes."
    AdminConfig.reset()
    raise # Re-throw to terminate the script
    
    
